import random
import string


def gen_hash(length=8):
    """
        generates a hash with ASCII letters and digits,
        always starts with a letter (for markdown usage).
    """
    assert length > 1
    random.seed()
    _hash_head = ''.join(random.sample(string.ascii_letters, 1))
    _hash_body = ''.join(random.sample(string.ascii_letters+string.digits, length-1))
    return _hash_head+_hash_body



from .utils import Dataset


class IAPS(Dataset):
    dataset_labels = ['vad']

    def __init__(self, root='external/IAPS/IAPS', **kwargs):
        if 'label_types' in kwargs.keys():
            print('only vad labels are available in IAPS/NAPS, please check your args.')
            raise NotImplementedError
        super().__init__(root=root, label_types='vad', **kwargs)

    def __readgt__(self, root):
        return super().__readgt__(root, spliter='\t', start_line=0, suffix='.jpg')

    def load_label(self, root):
        for x in self.__readgt__(root):
            y = [float(_str.replace('\n', '')) for _str in x[1:]]
            self.ys['vad'].append(y)


class NAPS(IAPS):
    def __init__(self, root='external/IAPS/NAPS', **kwargs):
        super().__init__(root, **kwargs)



from . import functional


class Train():
    def __init__(self, model, criterion, optimizer, cuda=True, check_type_tensor=False):
        self.model = model
        self.criterion = criterion
        self.optimizer = optimizer
        self.cuda = cuda
        self.check_type_tensor = check_type_tensor

    def __call__(self, batch, info=None):
        loss = functional.train_step(
            batch=batch,
            model=self.model,
            criterion=self.criterion,
            optimizer=self.optimizer,
            cuda=self.cuda,
            check_type_tensor=self.check_type_tensor,
        )
        return loss


class Validation():
    def __init__(self, model, criterion, cuda=True, check_type_tensor=False):
        self.model = model
        self.criterion = criterion
        self.cuda = cuda
        self.check_type_tensor = check_type_tensor

    def __call__(self, batch, info=None):
        loss = functional.validation_step(
            batch=batch,
            model=self.model,
            criterion=self.criterion,
            cuda=self.cuda,
            check_type_tensor=self.check_type_tensor,
        )
        return loss



def smooth(inputs, weight=0.7):

    inputs = [x[-1] if type(x) is tuple else x for x in inputs.history]  # for reading accuracy_module history

    for i in range(int(10*weight)+1):
        ret = []
        for idx, x in enumerate(inputs):
            if idx == 0 or idx == len(inputs)-1:
                ret.append(x)
            else:
                ret.append((inputs[idx-1]+x+inputs[idx+1])/3*weight+x*(1-weight))
        inputs = ret
    return ret



from .accuracy import Accuracy
from .looper import TrainLooper, ValLooper
from .step import Train, Validation
from .smooth import Smooth
from .dry import Dry
from .dataloaders import DataLoaders
from .time import Time
from .logger import Logger
from .hash import gen_hash

__all__ = ['Accuracy', 'TrainLooper', 'ValLooper', 'Train', 'Validation', 'Smooth', 'Dry', 'DataLoaders', 'Time', 'Logger', 'gen_hash']



import torch
import numpy as np
import random


def manual_seed(seed):
    torch.manual_seed(seed)  # cpu
    torch.cuda.manual_seed_all(seed)  # gpu
    np.random.seed(seed)  # numpy
    random.seed(seed)  # random and transforms
    torch.backends.cudnn.deterministic = True  # cudnn
    # https://zhuanlan.zhihu.com/p/76472385
    # torch.backends.cudnn.benchmark = False



from . import utils
from .emod import EMOd
from .emotion_roi import EmotionROI
from .fi import FI
from .iaps import IAPS, NAPS
from .ldl import FlickrLDL, TwitterLDL

__all__ = ['utils', 'FlickrLDL', 'TwitterLDL', 'IAPS', 'NAPS', 'EmotionROI', 'EMOd', 'FI']



from . import functional


class Smooth():
    def __init__(self, weight):
        self.weight = weight

    def __call__(self, inputs):
        return functional.smooth(
            inputs=inputs,
            weight=self.weight,
        )
